#!/usr/bin/env ruby

# Script to move 'WWW: url' from pkg-descr to Makefile
# this should greatly speed up index creation (with an
# additional change of it reading it from the Makefile
# instead of pkg-descr)

require 'byebug'
require 'open3'

def get_www(pkgdescr)
  File.readlines(pkgdescr).each do |line|
    if /\AWWW:.+/.match?(line)
      return line.strip.split(/\s+/).last
    end
  end
  return nil
end

def remove_www(pkgdescr)
  lines = File.readlines(pkgdescr)
              .reject{ |line| /\AWWW:.+/.match?(line) }

  last_index = 0
  lines.each_with_index do |line, index|
    last_index = index unless line.chomp.strip.empty?
  end

  File.open(pkgdescr,'wb'){|f| f.write(lines[0..last_index].join)}
end

def add_www(makefile, www)
  variables_before = %w{
    PORTNAME
    PORTVERSION
    DISTVERSIONPREFIX
    DISTVERSION
    DISTVERSIONSUFFIX
    PORTREVISION
    PORTEPOCH
    CATEGORIES
    MASTER_SITES}
  variables_after = %w{
    PKGNAMEPREFIX
    PKGNAMESUFFIX
    DISTANAME
    EXTRACT_SUFX
    DISTFILES
    DIST_SUBDIR
    EXTARCT_ONLY}

  lines = File.readlines makefile
  read_linenumber = ->(variable_name) {
    regex = /\A#{variable_name}.?=.*/
    if (linenumber = lines.index{ |line| line =~ regex })
      while /\\\z/.match?(lines[linenumber].chomp)
        linenumber += 1
      end
      return linenumber
    end
    return nil
  }

  get_positions = ->(variables) {
    result = {}
    variables.map do |variable|
      if (linenumber = read_linenumber[variable])
        result[variable] = linenumber
      end
    end
    return result.sort_by{ |k,v| v }.to_h
  }

  before = get_positions[variables_before]
  after  = get_positions[variables_after]

  insert_to = nil
  if before.any?
    insert_to    = before.values.last + 1
    add_new_line = false
  elsif after.any?
    insert_to     = after.values.first
    add_new_line = true
  end

  if insert_to
    lines.insert insert_to, "WWW=\t\t#{www}\n#{add_new_line ? "\n" : ""}"
    File.open(makefile,'wb'){|f| f.write(lines.join)}
  else
    puts "\tcould not update #{makefile} with #{www}"
  end
end


def query_var(port, variable)
  stdout, stderr, status = Open3.capture3(
    'make', '-C', port, '-V', variable
  )

  if !status.success?
    puts stderr
    raise 'failed to run make'
  end

  return stdout.chomp.strip
end

def update_port(port)
  makefile = File.join(port, 'Makefile')
  return unless File.exist?(makefile)

  get_descr = ->() {
    location = File.join(port, 'pkg-descr')
    if File.exist?(location)
      return true, location
    else
      location = query_var(port, 'DESCR')
      if File.exist?(location)
        return false, location
      end
    end
    return nil, nil
  }

  do_remove, pkgdescr = get_descr[]
  if pkgdescr.nil?
    puts "\tcould not find pkg-descr"
    return
  end

  if (www = get_www(pkgdescr))
    add_www(makefile, www)
    remove_www(pkgdescr) if do_remove
  else
    puts "\tNo www found for '#{port}'"
  end
end

if ARGV.none?
  puts "no arguments given"
  exit 1
end

ARGV.each do |arg|
  next unless File.directory?(arg)
  puts "Updating: #{arg}"
  update_port File.realpath(arg)
end
